/*
 * Trusted users list management. This used to be the core for the TPE 
 * patch. Now that Stephanie's design had changed, the trust list becomes 
 * more 'global'. The license says I have to leave the following comment 
 * in the source, so it stays -- most of the things there are obsolete
 * for this version of Stephanie, however.
 *
 * br1an <brian@innu.org>
 */

/*
 *  $Id: P54-06,v 1.16 1998/12/10 00:01:28 route Exp $
 *  Trusted path ACL implementation for OpenBSD 2.4
 *
 *  Copyright (c) 1998 route|daemon9 and Mike D. Schiffman
 *  All rights reserved.
 *  Originally published in Phrack Magazine (http://www.phrack.com).
 *
 *  Thanks to nirva for helping me choose an ADT.
 *  See <sys/kern_tpe.h> for more info.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/kern_acl.h>

/*
 * Initialize the trust list. (just fills it with invalid UIDs)
 */
void acl_init(void) {
	memset(acl, ACL_INITIALIZER, sizeof(uid_t) * ACL_SIZE);
	acl_users = 0;
	printf("Stephanie: ACL: Trusted users list initialized.\n");
}

/*
 * Add a UID to the trust list.
 */
int acl_add(uid_t uid) {
	int s = splhigh();

	/*
	 * Check if we have a full list.
	 */
	if (acl_users == ACL_SIZE) {
		splx(s);
		return (NACK);
	}

	/*
         * Don't add duplicates.
         */
	if ((acl_search(uid, 0, acl_users)) == NACK) {
		/*
	         * Add to the end of the list, then sort.
	         */
		acl_users++;
		acl[acl_users] = uid;
		acl_sort(0, acl_users);
	} else {
#ifdef STEPHANIE_VERBOSE
		uprintf("Stephanie: ACL: UID %u already in trust "
		    "list.\n", uid);
#endif
	}

	splx(s);
	return (ACK);
}

/*
 * Remove a UID from the trust list.
 */
int acl_remove(uid_t uid) {
	int n, s = splhigh();

	/*
	 * Check if we have an empty list.
	 */
	if (acl_users == 0) {
		splx(s);
		return (NACK);
	}

	if ((n = acl_search(uid, 0, acl_users)) != NACK) {
		/*
	         * Remove the UID from the trust list by marking the
		 * slot as unused and resort the list.
	         */
		acl[n] = ACL_INITIALIZER;
		acl_users--;
		acl_sort(0, acl_users);

		splx(s);
		return (ACK);
	}

	/*
         *  Not found.
         */
#ifdef STEPHANIE_VERBOSE
	uprintf("Stephanie: ACL: UID %u does not exist in trust list.\n",
	    uid);
#endif

	splx(s);
	return (NACK);
}

/*
 * Verify if a given UID is in the trust list.
 */
int acl_verify(uid_t uid) {
	if ((acl_search(uid, 0, acl_users)) != NACK)
		return (ACK);

	return (NACK);
}

/*
 * Sort the trust list.
 */
void acl_sort(int low, int high) {
	int i, j, n, s = splhigh();

	/*
         *  Standard insertion sort.
         */
	for (i = low + 1; i <= high; i++) {
		COMPSWAP(acl[low], acl[i]);
	}

	for (i = low + 2; i <= high; i++) {
		j = i;
		n = acl[i];
		while (LESS(n, acl[j - 1])) {
			acl[j] = acl[j - 1];
			j--;
		}
		acl[j] = n;
	}

	splx(s);
}

/*
 * Use binary search to search for a given UID in the trust list.
 */
int acl_search(uid_t uid, int low, int high) {
	int n, s = splhigh();

	/*
         *  Standard binary search.  XXX - should be iterative.
         */
	n = (low + high) / 2;

	if (low > high) {
		splx(s);
		return (NACK);
	}

	if (uid == acl[n]) {
		splx(s);
		return (n);
	}

	if (low == high) {
		splx(s);
		return (NACK);
	}

	if (LESS(uid, acl[n])) {
		splx(s);
		return (acl_search(uid, low, n - 1));
	}

	splx(s);
	return (acl_search(uid, n + 1, high));
}
