/*
 * Copyright (c) 2002 br1an <brian@innu.org>
 * All rights reserved.
 *
 * Official Stephanie homepage:
 *
 *	http://innu.org/~brian/Stephanie
 *
 * Shoutz go to shegget, S`, _eci, doe, jaf.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/kern_acl.h>
#include <sys/systm.h>
#include <sys/errno.h>

#include <sys/mount.h>
#include <sys/syscallargs.h>

int sys_aclctl(p, v, retval)
	struct proc *p;
	void *v;
	register_t *retval;
{
	struct sys_aclctl_args	/* {
		syscallarg(int) cmd;
		syscallarg(uid_t) uid;
	    	syscallarg(uid_t *) list;
	} */ *uap = v;
	register struct pcred *pc = p->p_cred;
	register int i;
	register uid_t *lp;

	/* 
	 * The only thing a non root user can do with this system call is
	 * verify if a given uid is in the trust list or not. This is 
	 * necessary for allowing trusted users normal access via kvm(3)
	 * and to ld.so environment variables. The status of ld.so is done
	 * via sysctl(3) now.  
	 */   
	if (suser(pc->pc_ucred, &p->p_acflag) &&
	    SCARG(uap, cmd) != ACL_VERIFY)
		return (EACCES);
	
	switch (SCARG(uap, cmd)) {
	case ACL_ADD:
		if (acl_add(SCARG(uap, uid)) == ACK) {
#ifdef STEPHANIE_VERBOSE
			uprintf("Stephanie: ACL: UID %u added to trust "
			    "list.\n", uid);
#endif
			return (0);
		}

		return (EINVAL);

	case ACL_REMOVE:
		if (acl_remove(SCARG(uap, uid)) == ACK) {
#ifdef STEPHANIE_VERBOSE
			printf("Stephanie: ACL: UID %u removed from trust"
			    " list.\n", uid);
#endif
			return (0);
		}

		return (EINVAL);

	case ACL_SHOW:
		lp = SCARG(uap, list);
		if (lp == NULL)
			return (EFAULT);

		for (i = 0; i < ACL_SIZE; i++)
			lp[i] = acl[i];

		return (0);

	case ACL_VERIFY:
		if (TRUSTED_USER(SCARG(uap, uid)))
			return(ACK);

		return(0);
	}

	return (EINVAL);
}
