/*
 * Copyright (c) 2002 br1an <brian@innu.org>
 * All rights reserved.
 *
 * Official Stephanie homepage:
 *
 *	http://innu.org/~brian/Stephanie
 *
 * Shoutz go to shegget, S`, _eci, doe, jaf.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/errno.h>
#include <sys/kern_sneek.h>
#include <sys/mount.h>
#include <sys/syscallargs.h>

int sys_sneekctl(p, v, retval)
	struct proc *p;
	void *v;
	register_t *retval;
{
	struct sys_sneekctl_args /* {
	        syscallarg(int) cmd;
	        syscallarg(int) flags;
	        syscallarg(int) maxargs;
	} */ *uap = v;

	int doflags, maxargs;
	extern unsigned int sneek_flags;
	extern unsigned int sneek_maxargs;

	/*
	 * Only the superuser is allowed to use this syscall.
	 */
	if (suser(p->p_cred->pc_ucred, &p->p_acflag))
		return(EPERM);

	/*
	 * Figure out what the task we need to perform is, and work 
	 * accordingly. If only 'maxargs' is set, don't return, just
	 * set the maxargs to that value. (OR'ing zero won't harm the
	 * value in sneek_flags)
	 */
	doflags = SCARG(uap, flags);
	maxargs = SCARG(uap, maxargs);
	if (!doflags && !maxargs &&
	    SCARG(uap, cmd) != SNEEK_SHOW_FLAGS &&
	    SCARG(uap, cmd) != SNEEK_SET_ON && 
	    SCARG(uap, cmd) != SNEEK_SET_OFF)
		return(EINVAL);

	switch (SCARG(uap, cmd)) {
		case SNEEK_SET_ON:
		case SNEEK_SET_OFF:
			if ((securelevel <= 0) || 
			    (SCARG(uap, cmd) == SNEEK_SET_ON)) {
				sneek_mode(SCARG(uap, cmd));
				return(0);
			}

			uprintf("Stephanie: Can't turn logging off in "
			    "multiuser mode.\n");

			return(EPERM);

		case SNEEK_ADD_FLAGS:
			sneek_flags |= doflags;
			sneek_maxargs = maxargs;

			return(0);

		case SNEEK_DEL_FLAGS:
			sneek_flags &= ~doflags;

			return(0);

		case SNEEK_SET_FLAGS:
			sneek_flags = doflags;
			sneek_maxargs = maxargs;

			return(0);

		case SNEEK_SHOW_FLAGS:
			sneek_show();

			return(0);
	}

	return (EINVAL);
}
