/*
 * Copyright (c) 2002 br1an <brian@innu.org>
 * All rights reserved.
 *
 * Official Stephanie homepage:
 *
 *	http://innu.org/~brian/Stephanie
 *
 * Shoutz go to shegget, S`, _eci, doe, jaf.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/param.h>
#include <sys/sysctl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <err.h>
#include <sysexits.h>
#include <syslog.h>

void usage(void) {
	extern char *__progname;

	(void) fprintf(stderr, "Stephanie: ld.so environment protection\n"
	    "Usage: %s <flag>\n"
	    "flags:\n"
	    "  e - enable\n"
	    "  d - disable\n"
	    "  s - show current status\n", __progname);
}

int main(int argc, char **argv) {
	int mib[2], newp, val, c;
	size_t len;
	extern char *__progname;

	if (argc != 2) {
		usage();
		exit(EX_USAGE);
	}

	if (strlen(argv[1]) != 1) {
		usage();
		exit(EX_USAGE);
	}

	c = *argv[1];
	openlog(__progname, LOG_PID, LOG_AUTH);

	switch (c) {
	case 'd':
	case 'e':
		mib[0] = CTL_KERN;
		mib[1] = KERN_LDSTRIP;

		newp = (c == 'e') ? 1 : 0;
		len = sizeof(val);

		if (sysctl(mib, 2, &val, &len, &newp, len) == -1)
			err(EX_OSERR, "sysctl");

		syslog(LOG_NOTICE, "ld.so environment protection %s.\n",
		    (c == 'e') ? "enabled" : "disabled");
		(void) printf("%s: ld.so environment protection %s.\n",
		    __progname, (c == 'e') ? "enabled" : "disabled");

		break;

	case 's':
		mib[0] = CTL_KERN;
		mib[1] = KERN_LDSTRIP;

		val = 0;
		len = sizeof(val);

		if (sysctl(mib, 2, &val, &len, NULL, 0) == -1)
			err(EX_OSERR, "sysctl");

		(void) printf("%s: ld.so environment protection is "
		    "currently %s.\n", __progname, val ? "on" : "off");

		return (0);

	default:
		usage();
		exit(EX_USAGE);
	}

	closelog();

	return (0);
}
