/*
 * Copyright (c) 2002 br1an <brian@innu.org>
 * All rights reserved.
 *
 * Official Stephanie homepage:
 *
 *	http://innu.org/~brian/Stephanie
 *
 * Shoutz go to shegget, S`, _eci, doe, jaf.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/kern_sneek.h>
#include <sys/syscall.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

int get_cmd(char *p) {
	if (!strcmp(p, "add")) return (SNEEK_ADD_FLAGS);
	if (!strcmp(p, "del")) return (SNEEK_DEL_FLAGS);
	if (!strcmp(p, "set")) return (SNEEK_SET_FLAGS);
	if (!strcmp(p, "show")) return (SNEEK_SHOW_FLAGS);
	if (!strcmp(p, "on")) return (SNEEK_SET_ON);
	if (!strcmp(p, "off")) return (SNEEK_SET_OFF);

	return (-1);
}

int parse(char *p) {
	int flags;

	flags = 0;
	while (*p) {
		switch (*p) {
			case 'a':
				flags |= SNEEK_LOG_ARGS;
				break;
			case 'b':
				flags |= SNEEK_LOG_BG;
				break;
			case 'p':
				flags |= SNEEK_LOG_PID;
				break;
			case 'u':
				flags |= SNEEK_LOG_UIDGID;
				break;
			case 'm':
				flags |= SNEEK_LOG_ARGS_MAX;
				break;
			case '0':
				return (0);
			default:
				goto bad;
		}

		p++;
	}

	return (flags);

bad:
	return (-1);
}

void usage(void) {
	extern char *__progname;

	(void) fprintf(stderr, "Usage: %s <cmd> [flags] [maxargs]\n"
	    "Read sneekctl(8) for more info.\n", __progname);

	exit(-1);
}

int main(int argc, char **argv) {
	int cmd, doflags, maxargs, i;

	/*
	 * We need superuser access.
	 */
	if (geteuid() || getuid()) {
		fprintf(stderr, "Superuser access required.\n");
		exit(1);
	}

	if ((argc < 2) || (argc > 4)) {
		usage();
		exit(0);
	}

	cmd = get_cmd(argv[1]);
	if (cmd == -1) {
		usage();
		exit(-1);
	}

	switch (cmd) {
		case SNEEK_SET_ON:
		case SNEEK_SET_OFF:
			i = syscall(SYS_sneekctl, cmd, 0, 0);
			if (i == -1) {
				perror("syscall");
				exit(-1);
			}

			return (0);

		case SNEEK_ADD_FLAGS:
		case SNEEK_DEL_FLAGS:
		case SNEEK_SET_FLAGS:
			doflags = parse(argv[2]);
			if (doflags == -1) {
				(void) fprintf(stderr, "Bad flags.\n");
				exit(-1);
			}

			maxargs = 0;
			if (argc == 4) maxargs = atoi(argv[3]);
			if (!maxargs && (doflags & SNEEK_LOG_ARGS_MAX) && 
			    cmd != SNEEK_DEL_FLAGS) {
				doflags &= ~SNEEK_LOG_ARGS_MAX;
				(void) printf("Number of arguments to log"
				    " not specified; removed flag.\n");
			}

			i = syscall(SYS_sneekctl, cmd, doflags, 
			    maxargs);
			if (i == -1) {
				perror("syscall");
				exit(-1);
			}

			return (0);

		case SNEEK_SHOW_FLAGS:
			i = syscall(SYS_sneekctl, SNEEK_SHOW_FLAGS, 0, 
			    0);
			if (i == -1) {
				fprintf(stderr, "Unable to get flags.\n");
				exit(-1);
			}

			return (0);
	}

	return (0);
}
