/*
 * Copyright (c) 2002 br1an <brian@innu.org>
 * All rights reserved.
 *
 * Official Stephanie homepage:
 *
 *	http://innu.org/~brian/Stephanie
 *
 * Shoutz go to shegget, S`, _eci, doe, jaf.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __KERN_ACL_H
#define __KERN_ACL_H

#ifdef _KERNEL
#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/ucred.h>
#include <sys/proc.h>
#endif /* _KERNEL */

#define	ACK			1	/* Positive acknowledgement */
#define	NACK			-1	/* Negative acknowledgement */

/*
 * Macros for the acl_sort function.
 */
#define	LESS(x, y)		(x < y)
#define SWAP(x, y)		(x ^= y, y ^= x, x ^= y)
#define COMPSWAP(x, y)		if (LESS(y, x)) SWAP(x, y)

/*
 * Commands for the aclctl syscall.
 */
#define ACL_ADD		1	/* Add an entry */
#define ACL_REMOVE	2	/* Delete an entry */
#define ACL_SHOW	3	/* Show the trust list */
#define ACL_VERIFY	4	/* Verify if a given UID is in the trust list */

/*
 * Size of the trust list.
 */
#define	ACL_SIZE	80	/* Shouldn't need to be larger */
#define ACL_INITIALIZER	-1	/* An invalid UID */

/*
 * Macro for verifying if a UID is trusted or not.
 */
#define	TRUSTED_USER(uid) (acl_verify(uid) == ACK)

uid_t acl[ACL_SIZE];		/* Trusted users list */
int acl_users;			/* Number of trusted users */

#ifdef LDSTRIP
int ld_check;			/* Check ld.so environment variables */
#endif /* LDSTRIP */

void	acl_init(void);
void	acl_sort(int, int);
int	acl_search(uid_t, int, int);
int	acl_add(uid_t);
int	acl_remove(uid_t);
int	acl_verify(uid_t);

#endif /* __KERN_ACL_H */
