/*
 * Copyright (c) 2002 br1an <brian@innu.org>
 * All rights reserved.
 *
 * Official Stephanie homepage:
 *
 *	http://innu.org/~brian/Stephanie
 *
 * Shoutz go to shegget, S`, _eci, doe, jaf.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __KERN_K5_H
#define __KERN_K5_H

#include <sys/types.h>
#include <sys/param.h>
#include <sys/exec.h>
#include <sys/namei.h>
#include <sys/queue.h>
#include <sys/stat.h>
#include <sys/syslimits.h>

#ifdef _KERNEL
	#include <sys/kernel.h>
	#include <sys/systm.h>
	#include <sys/errno.h>
	#include <sys/malloc.h>
	#include <sys/md5k.h>
	#include <sys/proc.h>

	/*
	 * Pointer to the head of the database.
	 */
	LIST_HEAD(k5_head_t, k5_info) k5_head;	

	/*
	 * Number of entries in the database.
	 */
	int k5_size;
#endif /* _KERNEL */

/*
 * Command types for the database management syscall.
 */
#define K5_ADD		1		/* Add an entry */
#define K5_DEL		2		/* Remove an entry */
#define K5_GETINFO	3		/* Get data for specified entry */
#define K5_GETOFFSET	4		/* Get offset for specified entry */

/*
 * Length of an MD5 checksum, including terminating NULL, as specified
 * in md5(3).
 */
#define	MD5_SUM_LEN	33

/*
 * Data structure of an entry in the k5 database. Fields are as follows:
 *	k5_list: Pointer to the database
 *	k5_inode: Inode number of the file
 *	k5_device: Device inode resides on
 *	k5_path: Full path of the file (from userland)
 *	k5_md5: MD5 checksum -- 33-byte ASCII string
 *	k5_matches: Number of valid matches against entry
 *	k5_mismatches: Number of invalid matches against entry
 */
struct k5_info {
	LIST_ENTRY(k5_info) k5_list;
	ino_t k5_inode; dev_t k5_device;
	char k5_path[PATH_MAX], k5_md5[33];
	u_quad_t k5_matches, k5_mismatches;
};

/*
 * Function prototypes for the kernel operations.
 */
#ifdef _KERNEL
void		k5_init(void);
int		k5_md5(struct exec_package *, char *);
struct k5_info *k5_search(ino_t, dev_t);
int		k5_verify(struct exec_package *);
int		k5_add(ino_t, dev_t, char *, char *);
void		k5_del(struct k5_info *);
#endif /* _KERNE_ */

#endif /* __KERN_K5_H */
